package com.wrongcoder.ld48.lowmemory.core;

import com.badlogic.gdx.Application;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.JsonReader;
import com.badlogic.gdx.utils.JsonValue;
import com.badlogic.gdx.utils.SerializationException;

public class Game extends com.badlogic.gdx.Game {

	private final Registry r;

	private String version = "";
	private String rawVersion = "";

	public Game(final Registry r) {
		this.r = r;
	}

	@Override
	public void create() {
		Gdx.app.setLogLevel(Application.LOG_ERROR);
		loadVersion();
		r.initialize();
		setScreen(new LoadingScreen(r).setNextScreen(new MainMenuScreen(r)));
	}

	public String getVersion() {
		return version;
	}

	public String getRawVersion() {
		return rawVersion;
	}

	private void loadVersion() {
		try {
			final FileHandle versionFile = Gdx.files.internal("version.json");
			final JsonValue versionRoot = new JsonReader().parse(versionFile);
			final String versionNumber = versionRoot.get("version").asString();
			final String buildNumber = versionRoot.get("buildNumber").asString();
			final String buildDate = versionRoot.get("buildDate").asString();
			final String commit = versionRoot.get("commit").asString();

			final String version;
			final String rawVersion;
			final String buildDateAddendum;

			if (buildDate != null && !buildDate.isEmpty() && !buildDate.contains("${")) {
				buildDateAddendum = " (" + buildDate + ")";
			} else {
				buildDateAddendum = "";
			}

			if (buildNumber != null && !buildNumber.isEmpty() && !buildNumber.contains("${")) {
				version = "Build " + buildNumber + buildDateAddendum;
				rawVersion = buildNumber;
			} else if (commit != null && !commit.isEmpty() && !commit.contains("${")) {
				version = "Commit " + commit + buildDateAddendum;
				rawVersion = commit;
			} else {
				version = "Version " + versionNumber + buildDateAddendum;
				rawVersion = version;
			}

			this.version = version;
			this.rawVersion = rawVersion;

		} catch (SerializationException e) {
			r.log.error("Exception", e);
			this.version = "(unknown version)";
			this.rawVersion = "unknown";
		}
	}

}
