package com.wrongcoder.ld48.lowmemory.core;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.wrongcoder.ld48.lowmemory.core.game.Level01;
import com.wrongcoder.ld48.lowmemory.core.game.Tutorial1Screen;

public class MainMenuScreen extends Screen {

	private Stage stage;
	private SpriteBatch batch;
	private BitmapFont pixelFont;
	private boolean gameWinFlag;

	public MainMenuScreen(final Registry r) {
		super(r);
	}

	@Override
	public void show() {
		stage = new Stage();
		Gdx.input.setInputProcessor(stage);

		batch = new SpriteBatch();
		pixelFont = r.assetManager.get(Fonts.pixelFont);
		gameWinFlag = r.getGameWinFlag();

		final Skin uiSkin = r.assetManager.get(AssetManager.ui, Skin.class);

		final TextButton instructionsButton = new TextButton("Tutorial", uiSkin);
		instructionsButton.setSize(300, 48);
		instructionsButton.setPosition(250, 225);
		instructionsButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				r.game.setScreen(new Tutorial1Screen(r));
			}
		});

		final TextButton startButton = new TextButton("Start", uiSkin);
		startButton.setSize(300, 48);
		startButton.setPosition(250, 165);
		startButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				r.game.setScreen(new Level01(r));
			}
		});

		final TextButton creditsButton = new TextButton("Credits", uiSkin);
		creditsButton.setSize(300, 48);
		creditsButton.setPosition(250, 105);
		creditsButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				final CreditsScreen creditsScreen = new CreditsScreen(r);
				r.game.setScreen(creditsScreen);
				creditsScreen.setNextScreen(MainMenuScreen.this);
			}
		});

		stage.addActor(instructionsButton);
		stage.addActor(startButton);
		//stage.addActor(creditsButton);
	}

	@Override
	public void hide() {
		batch.dispose();
		stage.dispose();
		Gdx.input.setInputProcessor(null);
	}

	@Override
	public void render(final float delta) {
		Gdx.gl.glClearColor(0, 0, 0, 0);
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
		stage.act(delta);
		stage.draw();

		batch.begin();
		if (!gameWinFlag) {
			drawCentred(pixelFont, batch, "Low Memory", Registry.WIDTH / 2, 490);
		} else {
			drawCentred(pixelFont, batch, "LOW MEMORY: Robot Invasion", Registry.WIDTH / 2, 490);
		}
		drawCentred(pixelFont, batch, "A Ludum Dare 27 compo entry", Registry.WIDTH / 2, 430);
		drawCentred(pixelFont, batch, "by wrongcoder", Registry.WIDTH / 2, 405);
		drawCentred(pixelFont, batch, "Written in 48 hours", Registry.WIDTH / 2, 345);

		batch.end();
	}

}
