package com.wrongcoder.ld48.lowmemory.core;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Net;
import com.badlogic.gdx.net.HttpParametersUtils;
import com.wrongcoder.ld48.lowmemory.core.game.AbstractLevelScreen;

import java.util.HashMap;
import java.util.Map;

public class ScoreTransmitter {

	private final Registry r;

	public ScoreTransmitter(final Registry r) {
		this.r = r;
	}

	/*
	 * - platform
	 * - level name
	 * - win / lose
	 * - number of ticks played
	 * - fuel remaining
	 *
	 */

	public void transmitStart(final AbstractLevelScreen level) {
		transmitData("start", level);
	}

	public void transmitLost(final AbstractLevelScreen level) {
		transmitData("lose", level);
	}

	public void transmitWon(final AbstractLevelScreen level) {
		transmitData("win", level);
	}

	private void transmitData(final String event, final AbstractLevelScreen level) {
		final Net.HttpRequest request = new Net.HttpRequest(Net.HttpMethods.GET);
		request.setUrl("http://wrongcoder.com/ld48/27/submit.php");

		final Map<String, String> requestParams = new HashMap<String, String>();
		requestParams.put("type", "score");
		requestParams.put("clientId", r.getClientId());
		requestParams.put("platform", r.platformId);
		requestParams.put("version", r.game.getRawVersion());
		requestParams.put("event", event);
		requestParams.put("level", level.getLevelId());
		requestParams.put("levelInstance", level.getInstanceId());
		requestParams.put("levelTime", String.valueOf(level.getLevelTime()));
		requestParams.put("fuel", String.valueOf(level.getRobot().getFuel()));
		requestParams.put("cheat", String.valueOf(level.getCheat()));
		request.setContent(HttpParametersUtils.convertHttpParameters(requestParams));

		r.log.info("Sending data: " + request.getContent());
		Gdx.net.sendHttpRequest(request, null);
	}

	public void transmitGameWon() {
		final Net.HttpRequest request = new Net.HttpRequest(Net.HttpMethods.GET);
		request.setUrl("http://wrongcoder.com/ld48/27/submit.php");

		final Map<String, String> requestParams = new HashMap<String, String>();
		requestParams.put("type", "win");
		requestParams.put("clientId", r.getClientId());
		requestParams.put("platform", r.platformId);
		requestParams.put("version", r.game.getRawVersion());
		request.setContent(HttpParametersUtils.convertHttpParameters(requestParams));

		r.log.info("Sending data: " + request.getContent());
		Gdx.net.sendHttpRequest(request, null);
	}

}
