package com.wrongcoder.ld48.lowmemory.core.game;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.InputMultiplexer;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.Box2DDebugRenderer;
import com.badlogic.gdx.physics.box2d.Contact;
import com.badlogic.gdx.physics.box2d.ContactImpulse;
import com.badlogic.gdx.physics.box2d.ContactListener;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.Manifold;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.World;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Array;
import com.wrongcoder.ld48.lowmemory.core.AssetManager;
import com.wrongcoder.ld48.lowmemory.core.Fonts;
import com.wrongcoder.ld48.lowmemory.core.MainMenuScreen;
import com.wrongcoder.ld48.lowmemory.core.Registry;
import com.wrongcoder.ld48.lowmemory.core.Screen;

public abstract class AbstractLevelScreen extends Screen {

	private World world;
	private Box2DDebugRenderer debugRenderer = new Box2DDebugRenderer();
	private final Array<Body> worldBodies = new Array<Body>(16);
	private float physicsTimeAccumulator = 0;
	public static final float metresInPixels = 32;
	public static final float pixelsInMetres = 1f / metresInPixels;
	public static final float timeStep = 1f / 20f;

	private SpriteBatch worldBatch;
	private ShapeRenderer worldShapeRenderer;
	private OrthographicCamera camera;
	protected Animation robotWait;
	protected Animation robotMove;
	protected Animation robotLeftThrust;
	protected Animation robotRightThrust;
	protected Animation exitDoorFrame;
	protected Animation exitDoorInside;
	protected Animation exitDoor2;
	protected Sprite leftCommandArrow;
	protected Sprite rightCommandArrow;
	protected Sprite upCommandArrow;
	protected Sprite canSprite;
	protected Sprite appleSprite;
	protected Sound thrusterSound;
	protected Sound pickupSound;
	protected Sound recordingSound;
	protected Sound warpoutSound;

	protected SpriteBatch hudBatch;
	protected ShapeRenderer hudShapeRenderer;
	protected BitmapFont pixelFont;
	protected Skin uiSkin;

	private Stage failedStage;
	private LevelWinPanel winPanel;

	private boolean debug = false;
	private int levelTime = 0;
	private Robot robot;
	private final Array<ExitDoorScreenObject> exitDoors = new Array<ExitDoorScreenObject>(4);

	private String instanceId;
	private State state = State.READY;
	private boolean cheat = false;
	private boolean reachedExit = false;
	private boolean reportedLose = false;
	private boolean reportedWin = false;
	private int commandsIx = 0;
	private Command[] commands = new Command[(int) (10 / timeStep)];

	public AbstractLevelScreen(final Registry r) {
		super(r);
	}

	protected abstract Screen makeRestartedScreen();

	@Override
	public void show() {
		world = new World(new Vector2(0, -9.8f), true);

		worldBatch = new SpriteBatch();
		worldShapeRenderer = new ShapeRenderer();
		camera = new OrthographicCamera();
		camera.setToOrtho(false, Registry.WIDTH * pixelsInMetres, Registry.HEIGHT * pixelsInMetres);
		camera.translate(0, -50 * pixelsInMetres);
		camera.update();
		worldBatch.setProjectionMatrix(camera.combined);
		worldShapeRenderer.setProjectionMatrix(camera.combined);

		final TextureAtlas textureAtlas = r.assetManager.get(AssetManager.textures);
		robotWait = new Animation(0.35f, textureAtlas.createSprites("sprites/robot-wait"), Animation.LOOP);
		robotMove = new Animation(0.125f, textureAtlas.createSprites("sprites/robot-move"), Animation.LOOP);
		robotLeftThrust = new Animation(0.1f, textureAtlas.createSprites("sprites/thrusters"), Animation.LOOP);
		final Array<Sprite> thrusterSprites = textureAtlas.createSprites("sprites/thrusters");
		for (final Sprite thrusterSprite : thrusterSprites) {
			thrusterSprite.flip(true, false);
		}
		robotRightThrust = new Animation(0.1f, thrusterSprites, Animation.LOOP);
		exitDoorFrame = new Animation(0.425f, textureAtlas.createSprites("sprites/exit-outline"), Animation.LOOP);
		exitDoorInside = new Animation(0.15f, textureAtlas.createSprites("sprites/exit-inside"), Animation.LOOP);
		exitDoor2 = new Animation(0, textureAtlas.createSprites("sprites/door2"), Animation.LOOP);
		leftCommandArrow = textureAtlas.createSprite("sprites/commandArrow");
		leftCommandArrow.setColor(1, 0.2f, 0.2f, 1);
		rightCommandArrow = textureAtlas.createSprite("sprites/commandArrow");
		rightCommandArrow.setColor(0.5f, 0.5f, 1, 1);
		rightCommandArrow.flip(true, false);
		upCommandArrow = textureAtlas.createSprite("sprites/commandArrow");
		upCommandArrow.setColor(1, 0.8f, 0.3f, 1);
		upCommandArrow.rotate90(true);
		canSprite = textureAtlas.createSprite("sprites/can");
		appleSprite = textureAtlas.createSprite("sprites/apple");

		thrusterSound = r.assetManager.get(AssetManager.thrusters);
		pickupSound = r.assetManager.get(AssetManager.pickup);
		recordingSound = r.assetManager.get(AssetManager.recording);
		warpoutSound = r.assetManager.get(AssetManager.warpout);

		initWorld();
		initLevel();

		hudBatch = new SpriteBatch();
		hudShapeRenderer = new ShapeRenderer();
		pixelFont = r.assetManager.get(Fonts.pixelFont);
		uiSkin = r.assetManager.get(AssetManager.ui, Skin.class);

		setupFailedStage();
		Gdx.input.setInputProcessor(new InputMultiplexer(failedStage, new GameScreenInputHandler()));
		instanceId = Registry.generateId();
		r.scoreTransmitter.transmitStart(this);
	}

	private void setupFailedStage() {
		failedStage = new Stage();

		final TextButton restartButton = new TextButton("Restart Level", uiSkin);
		restartButton.setSize(300, 48);
		restartButton.setPosition(250, 225);
		restartButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				r.game.setScreen(makeRestartedScreen());
			}
		});
		failedStage.addActor(restartButton);

		final TextButton quitButton = new TextButton("Quit", uiSkin);
		quitButton.setSize(300, 48);
		quitButton.setPosition(250, 165);
		quitButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				r.game.setScreen(new MainMenuScreen(r));
			}
		});
		failedStage.addActor(quitButton);
	}

	@Override
	public void hide() {
		worldBatch.dispose();
		worldShapeRenderer.dispose();
		world.dispose();
		hudBatch.dispose();
		hudShapeRenderer.dispose();
		failedStage.dispose();
		Gdx.input.setInputProcessor(null);

		if (winPanel != null) {
			winPanel.dispose();
		}
	}

	@Override
	public void render(final float delta) {
		physicsTimeAccumulator += delta;
		world.getBodies(worldBodies);

		Gdx.gl.glClearColor(0.12f, 0.12f, 0.12f, 1.0f);
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);

		final float alpha = Math.min(physicsTimeAccumulator / timeStep, 1);

		worldBatch.begin();
		for (final ExitDoorScreenObject exitDoor : exitDoors) {
			exitDoor.drawEarly(worldBatch);
		}
		for (final Body body : worldBodies) {
			final AbstractScreenObject object = (AbstractScreenObject) body.getUserData();
			object.draw(alpha, delta, worldBatch);
		}
		worldBatch.end();
		for (final Body body : worldBodies) {
			final AbstractScreenObject object = (AbstractScreenObject) body.getUserData();
			object.draw(alpha, delta, worldShapeRenderer);
		}

		hudBatch.begin();
		drawShadowed(pixelFont, hudBatch, "FUEL", 10, 590);
		drawShadowed(pixelFont, hudBatch, "TIME " + formatLevelTime(levelTime), 694, 590);
		drawShadowedCentred(pixelFont, hudBatch, getLevelTitle(), Registry.WIDTH / 2, 579);
		if (debug) {
			drawShadowed(pixelFont, hudBatch, "FPS: " + Gdx.graphics.getFramesPerSecond(), 10, 560);
			drawShadowed(pixelFont, hudBatch, String.valueOf(robot.getFuel()), 200, 590);
			drawShadowed(pixelFont, hudBatch, "(" + Gdx.input.getX() + "," + (550 - Gdx.input.getY()) + ")", 10, 530);
		}
		if (robot.getFuel() > 0) {
			if (state == State.READY) {
				drawShadowedCentred(pixelFont, hudBatch, "RECORD TRANSMISSION", Registry.WIDTH / 2, 520);
			}
			if (state == State.RECORD && levelTime % 20 < 12) {
				drawShadowedCentred(pixelFont, hudBatch, "RECORDING 10 SECONDS", Registry.WIDTH / 2, 520);
			}
			if (state == State.RECORD && levelTime % 20 == 3) {
				recordingSound.play(0.2f);
			}
		}
		hudBatch.end();

		if (debug) {
			debugRenderer.render(world, camera.combined);
		}

		hudShapeRenderer.begin(ShapeRenderer.ShapeType.Filled);
		// Fuel
		hudShapeRenderer.setColor(0, 0.4f, 0.2f, 1);
		hudShapeRenderer.rect(67, 572, 102, 12);
		hudShapeRenderer.setColor(0, 0.2f, 0.05f, 1);
		hudShapeRenderer.rect(68, 573, 100, 10);
		hudShapeRenderer.setColor(0, 0.75f, 0.2f, 1);
		hudShapeRenderer.rect(68, 573, 100 * robot.getFuelLevel(), 10);
		// Command bar
		hudShapeRenderer.setColor(0, 0, 0, 1);
		hudShapeRenderer.rect(0, 0, Registry.WIDTH, 50);
		hudShapeRenderer.setColor(0, 0.7f, 0, 1);
		hudShapeRenderer.rect(commandsIx * 4 + 1, 10, 1, 30);
		hudShapeRenderer.end();

		renderCommandBar();

		if (physicsTimeAccumulator > timeStep && !reachedExit) {
			physicsTimeAccumulator -= timeStep;
			levelTime += 1;
			updateWorld();
		}

		if (reachedExit) {
			if (winPanel == null) {
				winPanel = new LevelWinPanel(r, this);
			}
			robot.resetThrusters();
			reportWin();
			winPanel.render(delta);
		}
		else if (robot.getFuel() <= 0) {
			hudBatch.begin();
			drawShadowedCentred(pixelFont, hudBatch, "Out of fuel", Registry.WIDTH / 2, 350);
			hudBatch.end();
			reportLose();
			failedStage.draw();
		}
	}

	private void renderCommandBar() {
		hudBatch.begin();
		if (state != State.DIRECT) {
			for (int ix = 0; ix < commands.length; ix++) {
				final Command command = commands[ix];
				final int x = ix * 4 - 2;
				final int y = 20;

				if (command == Command.L) {
					leftCommandArrow.setPosition(x, y);
					leftCommandArrow.draw(hudBatch);
				}
				else if (command == Command.R) {
					rightCommandArrow.setPosition(x, y);
					rightCommandArrow.draw(hudBatch);
				}
				else if (command == Command.U) {
					upCommandArrow.setPosition(x, y);
					upCommandArrow.draw(hudBatch);
				}
			}
		} else {
			drawCentred(pixelFont, hudBatch, "--- CHEAT MODE --- DIRECT CONTROL --- CHEAT MODE ---", Registry.WIDTH / 2, 30);
		}
		hudBatch.end();
	}

	private void updateWorld() {
		switch (state) {
			case PAUSE:
				break;
			case READY:
				checkStartRecording();
				break;
			case RECORD:
				recordRobotControls();
				break;
			case PLAY:
				sendRobotControls();
				break;
			case DIRECT:
				directControl();
				break;
		}

		world.step(timeStep, 6, 2);
		for (final Body body : worldBodies) {
			final AbstractScreenObject object = (AbstractScreenObject) body.getUserData();
			object.move(body.getPosition());
		}
	}

	private void initWorld() {
		world.setContactListener(new LevelScreenContactListener());
		createPlatformInWorld(camera.viewportWidth / 2, 0, camera.viewportWidth, 0.25f);
		createPlatformInWorld(camera.viewportWidth / 2, camera.viewportHeight, camera.viewportWidth, 0.25f);
		createPlatformInWorld(-0.255f, camera.viewportHeight / 2, 0.25f, camera.viewportHeight);
		createPlatformInWorld(camera.viewportWidth + 0.255f, camera.viewportHeight / 2, 0.25f, camera.viewportHeight);
	}

	/** Create a Robot and one or more Exit Doors */
	protected abstract void initLevel();

	/** Create the next level */
	protected abstract Screen createNextLevel();

	public int getScore() {
		return Math.max(0, 20 * robot.getFuel() + (600 - levelTime));
	}

	private void createPlatformInWorld(final float x, final float y, final float width, final float height) {
		final BodyDef bodyDef = new BodyDef();
		final FixtureDef fixtureDef = new FixtureDef();
		final PolygonShape polygonShape = new PolygonShape();

		bodyDef.type = BodyDef.BodyType.StaticBody;
		bodyDef.position.set(x, y);
		bodyDef.fixedRotation = true;
		polygonShape.setAsBox(width, height);
		fixtureDef.shape = polygonShape;
		fixtureDef.friction = 1;
		fixtureDef.restitution = 0;
		final Body groundBody = world.createBody(bodyDef);
		groundBody.createFixture(fixtureDef);
		final PlatformScreenObject ground = new PlatformScreenObject(width, height);
		groundBody.setUserData(ground);
		ground.move(groundBody.getPosition());

		polygonShape.dispose();
	}

	protected void createPlatform(final float xPixels, final float yPixels, final float widthPixels, final float heightPixels) {
		createPlatformInWorld(xPixels * pixelsInMetres, yPixels * pixelsInMetres, widthPixels * pixelsInMetres, heightPixels * pixelsInMetres);
	}

	protected void createRobot(final float xPixels, final float yPixels, final int fuel) {
		if (robot != null) {
			throw new IllegalStateException("Robot must be unique");
		}

		final BodyDef bodyDef = new BodyDef();
		final FixtureDef fixtureDef = new FixtureDef();
		final PolygonShape polygonShape = new PolygonShape();

		bodyDef.type = BodyDef.BodyType.DynamicBody;
		bodyDef.position.set(xPixels * pixelsInMetres, yPixels * pixelsInMetres);
		bodyDef.linearDamping = 0.8f;
		bodyDef.fixedRotation = true;
		polygonShape.setAsBox(12 * pixelsInMetres, 16 * pixelsInMetres);
		fixtureDef.shape = polygonShape;
		fixtureDef.friction = 0.5f;
		fixtureDef.restitution = 0;
		fixtureDef.density = 4000;
		final RobotScreenObject robotScreenObject = new RobotScreenObject(robotWait, robotMove, robotLeftThrust, robotRightThrust, pickupSound, thrusterSound);
		final Body robotBody = world.createBody(bodyDef);
		robotBody.createFixture(fixtureDef);
		robotBody.setUserData(robotScreenObject);
		robotScreenObject.move(robotBody.getPosition());
		robot = new Robot(robotBody, robotScreenObject, fuel);

		polygonShape.dispose();
	}

	protected void createExitDoor(final int x, final int y) {
		final BodyDef bodyDef = new BodyDef();
		final FixtureDef fixtureDef = new FixtureDef();
		final PolygonShape polygonShape = new PolygonShape();

		bodyDef.type = BodyDef.BodyType.StaticBody;
		bodyDef.position.set(x * pixelsInMetres, y * pixelsInMetres);
		bodyDef.fixedRotation = true;
		polygonShape.setAsBox(6 * pixelsInMetres, 6 * pixelsInMetres);
		fixtureDef.shape = polygonShape;
		fixtureDef.isSensor = true;
		final ExitDoorScreenObject exitDoor = new ExitDoorScreenObject(exitDoorFrame, exitDoorInside);
		final Body exitDoorBody = world.createBody(bodyDef);
		exitDoorBody.createFixture(fixtureDef);
		exitDoorBody.setUserData(exitDoor);
		exitDoor.move(exitDoorBody.getPosition());

		exitDoors.add(exitDoor);
		polygonShape.dispose();
	}

	protected void createExitDoor2(final int x, final int y) {
		final BodyDef bodyDef = new BodyDef();
		final FixtureDef fixtureDef = new FixtureDef();
		final PolygonShape polygonShape = new PolygonShape();

		bodyDef.type = BodyDef.BodyType.StaticBody;
		bodyDef.position.set(x * pixelsInMetres, y * pixelsInMetres);
		bodyDef.fixedRotation = true;
		polygonShape.setAsBox(6 * pixelsInMetres, 6 * pixelsInMetres);
		fixtureDef.shape = polygonShape;
		fixtureDef.isSensor = true;
		final ExitDoor2ScreenObject exitDoor = new ExitDoor2ScreenObject(exitDoor2);
		final Body exitDoorBody = world.createBody(bodyDef);
		exitDoorBody.createFixture(fixtureDef);
		exitDoorBody.setUserData(exitDoor);
		exitDoor.move(exitDoorBody.getPosition());

		exitDoors.add(exitDoor);
		polygonShape.dispose();
	}

	protected void createCanPickup(final int x, final int y) {
		final BodyDef bodyDef = new BodyDef();
		final FixtureDef fixtureDef = new FixtureDef();
		final PolygonShape polygonShape = new PolygonShape();

		bodyDef.type = BodyDef.BodyType.StaticBody;
		bodyDef.position.set(x * pixelsInMetres, y * pixelsInMetres);
		bodyDef.fixedRotation = true;
		polygonShape.setAsBox(6 * pixelsInMetres, 8 * pixelsInMetres);
		fixtureDef.shape = polygonShape;
		fixtureDef.isSensor = true;
		final CanPickupScreenObject canScreenObject = new CanPickupScreenObject(canSprite);
		final Body canBody = world.createBody(bodyDef);
		canBody.createFixture(fixtureDef);
		canBody.setUserData(canScreenObject);
		canScreenObject.move(canBody.getPosition());

		polygonShape.dispose();
	}

	protected void createApplePickup(final int x, final int y) {
		final BodyDef bodyDef = new BodyDef();
		final FixtureDef fixtureDef = new FixtureDef();
		final PolygonShape polygonShape = new PolygonShape();

		bodyDef.type = BodyDef.BodyType.StaticBody;
		bodyDef.position.set(x * pixelsInMetres, y * pixelsInMetres);
		bodyDef.fixedRotation = true;
		polygonShape.setAsBox(6 * pixelsInMetres, 8 * pixelsInMetres);
		fixtureDef.shape = polygonShape;
		fixtureDef.isSensor = true;
		final ApplePickupScreenObject appleScreenObject = new ApplePickupScreenObject(appleSprite);
		final Body appleBody = world.createBody(bodyDef);
		appleBody.createFixture(fixtureDef);
		appleBody.setUserData(appleScreenObject);
		appleScreenObject.move(appleBody.getPosition());

		polygonShape.dispose();
	}

	private void checkStartRecording() {
		final Input input = Gdx.app.getInput();
		if (input.isKeyPressed(Input.Keys.LEFT) || input.isKeyPressed(Input.Keys.RIGHT) || input.isKeyPressed(Input.Keys.UP) || input.isKeyPressed(Input.Keys.SPACE)) {
			commandsIx = 0;
			state = State.RECORD;
		}
	}

	@SuppressWarnings("UnnecessaryReturnStatement")
	private void recordRobotControls() {
		final Input input = Gdx.app.getInput();

		if (input.isKeyPressed(Input.Keys.ESCAPE)) {
			for (int ix = 0; ix < commands.length; ix++) {
				commands[ix] = null;
			}
			commandsIx = 0;
			state = State.READY;
			return;
		}

		if (input.isKeyPressed(Input.Keys.LEFT) && input.isKeyPressed(Input.Keys.RIGHT)) {
			commands[commandsIx] = null;
		}
		else if (input.isKeyPressed(Input.Keys.UP)) {
			commands[commandsIx] = Command.U;
		}
		else if (input.isKeyPressed(Input.Keys.LEFT)) {
			commands[commandsIx] = Command.L;
		}
		else if (input.isKeyPressed(Input.Keys.RIGHT)) {
			commands[commandsIx] = Command.R;
		} else {
			commands[commandsIx] = null;
		}

		commandsIx += 1;
		if (commandsIx >= commands.length) {
			commandsIx = 0;
			state = State.PLAY;
			robot.startPlayback();
		}
	}

	@SuppressWarnings("UnnecessaryReturnStatement")
	private void sendRobotControls() {
		final Command command = commands[commandsIx];

		robot.resetThrusters();

		if (command == Command.U) {
			robot.fireLeftThruster();
			robot.fireRightThruster();
		}
		else if (command == Command.L) {
			robot.fireRightThruster();
		}
		else if (command == Command.R) {
			robot.fireLeftThruster();
		}

		commandsIx += 1;
		if (commandsIx >= commands.length) {
			commandsIx = 0;
			for (int ix = 0; ix < commands.length; ix++) {
				commands[ix] = null;
			}
			state = State.READY;
			robot.resetThrusters();
			robot.stopPlayback();
		}
	}

	private void directControl() {
		// Cheat Mode

		final Input input = Gdx.app.getInput();

		robot.resetThrusters();

		if (input.isKeyPressed(Input.Keys.LEFT) && input.isKeyPressed(Input.Keys.RIGHT)) {
			robot.fireLeftThruster();
			robot.fireRightThruster();
		}
		else if (input.isKeyPressed(Input.Keys.UP)) {
			robot.fireLeftThruster();
			robot.fireRightThruster();
		}
		else if (input.isKeyPressed(Input.Keys.LEFT)) {
			robot.fireRightThruster();
		}
		else if (input.isKeyPressed(Input.Keys.RIGHT)) {
			robot.fireLeftThruster();
		}
	}

	// String.format missing in GWT
	private static String formatLevelTime(final int levelTime) {
		final int levelSeconds = (int) (levelTime * timeStep);

		if (levelSeconds < 10) {
			return "  " + levelSeconds;
		}
		if (levelSeconds < 100) {
			return " " + levelSeconds;
		}
		if (levelSeconds > 999) {
			return "---";
		}
		return String.valueOf(levelSeconds);
	}

	private void reportWin() {
		if (reportedWin) {
			return;
		}
		r.scoreTransmitter.transmitWon(this);
		robot.warp();
		warpoutSound.play(0.5f);
		reportedWin = true;
	}

	private void reportLose() {
		if (reportedLose) {
			return;
		}
		r.scoreTransmitter.transmitLost(this);
		reportedLose = true;
	}

	public Robot getRobot() {
		return robot;
	}

	protected State getState() {
		return state;
	}

	public int getLevelTime() {
		return levelTime;
	}

	public abstract String getLevelId();

	public abstract String getLevelTitle();

	public String getInstanceId() {
		return instanceId;
	}

	public boolean getCheat() {
		return cheat;
	}

	private class GameScreenInputHandler extends InputAdapter {
		@Override
		public boolean keyUp(final int keyCode) {
			switch (keyCode) {
				case Input.Keys.F2:
					debug = !debug;
					return true;
				case Input.Keys.F4:
					if (debug) {
						state = State.DIRECT;
						cheat = true;
						robot.startPlayback();
						return true;
					} else {
						return false;
					}
				case Input.Keys.ESCAPE:
					if (debug) {
						robot.setFuel(0);
						return true;
					} else {
						return false;
					}
			}
			return false;
		}
	}

	public class LevelScreenContactListener implements ContactListener {

		@Override
		public void beginContact(final Contact contact) {
			final AbstractScreenObject objectA = (AbstractScreenObject) contact.getFixtureA().getBody().getUserData();
			final AbstractScreenObject objectB = (AbstractScreenObject) contact.getFixtureB().getBody().getUserData();

			//final RobotScreenObject robotScreenObject;
			final AbstractScreenObject otherScreenObject;

			if (objectA instanceof RobotScreenObject) {
				//robotScreenObject = (RobotScreenObject) objectA;
				otherScreenObject = objectB;
			}
			else if (objectB instanceof RobotScreenObject) {
				//robotScreenObject = (RobotScreenObject) objectB;
				otherScreenObject = objectA;
			}
			else {
				return;
			}

			if (otherScreenObject instanceof ExitDoorScreenObject) {
				reachedExit = true;
			}
			if (otherScreenObject instanceof AbstractPickupScreenObject) {
				robot.consume((AbstractPickupScreenObject) otherScreenObject);
			}
		}

		@Override
		public void endContact(final Contact contact) {
		}

		@Override
		public void preSolve(final Contact contact, final Manifold oldManifold) {
		}

		@Override
		public void postSolve(final Contact contact, final ContactImpulse impulse) {
		}
	}

	public enum State { PAUSE, READY, RECORD, PLAY, DIRECT, WON }

	public enum Command { L, R, U }

}
