package com.wrongcoder.ld48.lowmemory.core.game;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.wrongcoder.ld48.lowmemory.core.AssetManager;
import com.wrongcoder.ld48.lowmemory.core.Fonts;
import com.wrongcoder.ld48.lowmemory.core.MainMenuScreen;
import com.wrongcoder.ld48.lowmemory.core.Registry;
import com.wrongcoder.ld48.lowmemory.core.Screen;

public class LevelWinPanel {

	private final int score;
	private final Stage stage;
	private final BitmapFont font;
	private final SpriteBatch batch;

	public LevelWinPanel(final Registry r, final AbstractLevelScreen level) {
		this.score = level.getScore();

		this.stage = new Stage();

		font = r.assetManager.get(Fonts.pixelFont);
		batch = new SpriteBatch();

		final Skin uiSkin = r.assetManager.get(AssetManager.ui);

		final TextButton nextLevelButton = new TextButton("Continue", uiSkin);
		nextLevelButton.setSize(300, 48);
		nextLevelButton.setPosition(250, 275);
		nextLevelButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				final Screen nextLevel = level.createNextLevel();
				r.game.setScreen(nextLevel);
			}
		});
		stage.addActor(nextLevelButton);

		final TextButton restartButton = new TextButton("Try again", uiSkin);
		restartButton.setSize(300, 48);
		restartButton.setPosition(250, 220);
		restartButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				r.game.setScreen(level.makeRestartedScreen());
			}
		});
		stage.addActor(restartButton);

		final TextButton quitButton = new TextButton("Quit", uiSkin);
		quitButton.setSize(300, 48);
		quitButton.setPosition(250, 165);
		quitButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				r.game.setScreen(new MainMenuScreen(r));
			}
		});
		stage.addActor(quitButton);

		Gdx.input.setInputProcessor(stage);
	}

	public void dispose() {
		stage.dispose();
		batch.dispose();
		Gdx.input.setInputProcessor(null);
	}

	public void render(final float delta) {
		batch.begin();
		Screen.drawShadowedCentred(font, batch, "Robot has escaped", Registry.WIDTH / 2, 450);
		Screen.drawShadowedCentred(font, batch, "Score: " + score, Registry.WIDTH / 2, 400);
		batch.end();

		stage.draw();
	}

}
