package com.wrongcoder.ld48.lowmemory.core.game;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;

public class Robot {

	public final Body body;
	public final RobotScreenObject view;

	public final int maxFuel = (int) (10 / AbstractLevelScreen.timeStep);
	private int fuel = (int) (3 / AbstractLevelScreen.timeStep);

	private final float thrust = 30000 * AbstractLevelScreen.timeStep;

	public Robot(final Body body, final RobotScreenObject view, final int fuel) {
		this.body = body;
		this.view = view;
		this.fuel = fuel;
	}

	public void startPlayback() {
		view.setPlayback(true);
	}

	public void stopPlayback() {
		view.setPlayback(false);
	}

	public void resetThrusters() {
		view.resetThrusters();
	}

	public void fireRightThruster() {
		if (fuel > 0) {
			fuel -= 1;
			body.applyLinearImpulse(new Vector2(-thrust, thrust), body.getWorldCenter(), true);
			view.setRightThruster();
		} else {
			view.setHaveFuel(false);
		}
	}

	public void fireLeftThruster() {
		if (fuel > 0) {
			fuel -= 1;
			body.applyLinearImpulse(new Vector2(thrust, thrust), body.getWorldCenter(), true);
			view.setLeftThrusters();
		} else {
			view.setHaveFuel(false);
		}
	}

	public int getFuel() {
		return fuel;
	}

	public float getFuelLevel() {
		return (float) fuel / maxFuel;
	}

	public void setFuel(final int fuel) {
		this.fuel = fuel;
	}

	public void consume(final AbstractPickupScreenObject pickup) {
		if (pickup.isAvailable()) {
			view.consumeFuel();
			fuel = Math.min(maxFuel, fuel + pickup.consume());
		}
	}

	public void warp() {
		view.warp();
	}

}
