package com.wrongcoder.ld48.lowmemory.core.game;

import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;

public class RobotScreenObject extends AbstractScreenObject {

	private final Animation waitAnimation;
	private final Animation moveAnimation;
	private final Animation leftThrustersAnimation;
	private final Animation rightThrustersAnimation;
	private final Sound consumeFuelSound;
	private final Sound thrusterSound;
	private final float width, height;
	private final float offsetX, offsetY;
	private boolean haveFuel = true;
	private boolean moving = false;
	private boolean warped = false;
	private boolean leftThruster = false;
	private boolean rightThruster = false;

	private float animationStateTime = 0;

	public RobotScreenObject(final Animation waitAnimation, final Animation moveAnimation, final Animation leftThrustersAnimation, final Animation rightThrustersAnimation, final Sound consumeFuelSound, final Sound thrusterSound) {
		this.waitAnimation = waitAnimation;
		this.moveAnimation = moveAnimation;
		this.leftThrustersAnimation = leftThrustersAnimation;
		this.rightThrustersAnimation = rightThrustersAnimation;
		this.consumeFuelSound = consumeFuelSound;
		this.thrusterSound = thrusterSound;
		width = 32 * AbstractLevelScreen.pixelsInMetres;
		height = 32 * AbstractLevelScreen.pixelsInMetres;
		offsetX = width / 2f;
		offsetY = height / 2f;
	}

	public void setPlayback(final boolean playback) {
		this.moving = playback;
	}

	public void setHaveFuel(final boolean haveFuel) {
		this.haveFuel = haveFuel;
	}

	public void resetThrusters() {
		leftThruster = false;
		rightThruster = false;
	}

	public void setLeftThrusters() {
		this.leftThruster = true;
	}

	public void setRightThruster() {
		this.rightThruster = true;
	}

	@Override
	public void draw(final float alpha, final float delta, final SpriteBatch batch) {
		if (warped) {
			return;
		}

		final float x = (alpha * nextX) + ((1 - alpha) * prevX);
		final float y = (alpha * nextY) + ((1 - alpha) * prevY);

		animationStateTime += delta;
		if (!moving) {
			batch.draw(waitAnimation.getKeyFrame(animationStateTime), x - offsetX, y - offsetY, width, height);
		} else {
			if (haveFuel) {
				batch.draw(moveAnimation.getKeyFrame(animationStateTime), x - offsetX, y - offsetY, width, height);
			} else {
				batch.draw(moveAnimation.getKeyFrame(0), x - offsetX, y - offsetY, width, height);
			}
			if (leftThruster) {
				batch.draw(leftThrustersAnimation.getKeyFrame(animationStateTime), x - offsetX, y - offsetY, width, height);
			}
			if (rightThruster) {
				batch.draw(rightThrustersAnimation.getKeyFrame(animationStateTime), x - offsetX, y - offsetY, width, height);
			}

			if (leftThruster && rightThruster) {
				thrusterSound.play(0.285f);
			} else if (leftThruster || rightThruster) {
				thrusterSound.play(0.15f);
			}
		}
	}

	@Override
	public void draw(final float alpha, final float delta, final ShapeRenderer shapeRenderer) {
	}

	public void consumeFuel() {
		consumeFuelSound.play();
	}

	public void warp() {
		warped = true;
	}

}
