package com.wrongcoder.ld48.lowmemory.core.game;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.wrongcoder.ld48.lowmemory.core.AssetManager;
import com.wrongcoder.ld48.lowmemory.core.Fonts;
import com.wrongcoder.ld48.lowmemory.core.Registry;
import com.wrongcoder.ld48.lowmemory.core.Screen;

public class Tutorial1Screen extends Screen {

	private Stage stage;
	private SpriteBatch batch;
	private ShapeRenderer shapeRenderer;
	private BitmapFont font;

	public Tutorial1Screen(final Registry r) {
		super(r);
	}

	@Override
	public void show() {
		stage = new Stage();
		batch = new SpriteBatch();
		shapeRenderer = new ShapeRenderer();

		font = r.assetManager.get(Fonts.pixelFont);
		final Skin uiSkin = r.assetManager.get(AssetManager.ui);

		final TextButton okButton = new TextButton("OK", uiSkin);
		okButton.setSize(300, 48);
		okButton.setPosition(250, 175);
		okButton.addListener(new ChangeListener() {
			@Override
			public void changed(final ChangeEvent event, final Actor actor) {
				r.game.setScreen(new Tutorial2Screen(r));
			}
		});
		stage.addActor(okButton);

		Gdx.input.setInputProcessor(stage);
	}

	@Override
	public void hide() {
		stage.dispose();
		batch.dispose();
		shapeRenderer.dispose();
	}

	@Override
	public void render(final float delta) {
		Gdx.gl.glClearColor(0.12f, 0.12f, 0.12f, 1);
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);

		shapeRenderer.begin(ShapeRenderer.ShapeType.Filled);
		shapeRenderer.setColor(0, 0, 0, 0.8f);
		shapeRenderer.rect(137, 148, 530, 350);
		shapeRenderer.setColor(1, 1, 1, 1);
		shapeRenderer.rect(135, 150, 530, 350);
		shapeRenderer.setColor(0.8f, 0.8f, 0.8f, 1);
		shapeRenderer.rect(137, 150, 528, 348);
		shapeRenderer.end();

		final float yBase = 420;
		final float ySpace = 22;

		batch.begin();

		font.setColor(0, 0, 0, 1);
		drawCentred(font, batch, "ERROR REPORT", 400, 460);
		drawCentred(font, batch, "____________________", 400, 450);
		font.draw(batch, "Robot:", 255, yBase);
		font.draw(batch, "Fuel level:", 303, yBase - ySpace);
		font.draw(batch, "Last contact:", 303, yBase - ySpace * 2);
		font.draw(batch, "Controller:", 255, yBase - ySpace * 3);
		font.draw(batch, "Mem free:", 303, yBase - ySpace * 4);

		font.setColor(0.75f, 0, 0, 1);
		font.draw(batch, "critical", 447, yBase - ySpace);
		font.draw(batch, "never", 471, yBase - ySpace * 2);
		font.draw(batch, "10 seconds", 423, yBase - ySpace * 4);
		drawCentred(font, batch, "System Failure", 400, yBase - ySpace * 7);
		drawCentred(font, batch, "Initiate Manual Override", 400, yBase - ySpace * 8);

		font.setColor(0, 0, 0, 1);
		font.draw(batch, "1/2", 615, 185);

		batch.end();

		stage.draw();
	}

}
